--
-- FactionDiscount_AutoRepair
-- Author: Nafe
-- Guild: Clan Blackhoof
-- Server: Cenarion Circle
--

local _G = getfenv(0)

local FD_AR = {}
_G.FactionDiscount_AutoRepair = FD_AR

--stack references for globals
local GetRealmName = _G.GetRealmName
local UnitName = _G.UnitName

local UnitReaction = _G.UnitReaction

local GetMoney = _G.GetMoney
local GetRepairAllCost = _G.GetRepairAllCost

local CanMerchantRepair = _G.CanMerchantRepair
local RepairAllItems = _G.RepairAllItems

local strlower = _G.strlower

local strgsub = _G.string.gsub

----Saved variables----
--Set at VARIABLES_LOADED event
local FD_AR_saved = {}

----Local variables----
local txt2threshold = {
   ["always"]     = 4,
   ["honored"]    = 6,
   ["revered"]    = 7,
   ["exalted"]    = 8,
}
local threshold2txt = {
   [4] = "Always",
   [6] = "Honored",
   [7] = "Revered",
   [8] = "Exalted",
}
local discount2txt = {
   [4] = "Neutral",
   [5] = "Friendly or Unfactioned",
   [6] = "Honored",
   [7] = "Revered",
   [8] = "Exalted",
}
local default_threshold = 8

local slash_subcommands_info = {
   {
      ["name"] = "",
      ["desc"] = "This print-out!",
   },
   {
      ["name"] = "|cff00ff00enable|r",
      ["desc"] = "Enables autorepairing at merchants for this character.",
   },
   {
      ["name"] = "|cffff0000disable|r",
      ["desc"] = "Disables autorepairing at merchants for this character.",
   },
   {
      ["name"] = "|cff0000ffstatus|r",
      ["desc"] = "Displays enable state and faction discount threshold for this character.",
   },
   {
      ["name"] = "|cffffff00always | honored | revered | exalted|r",
      ["desc"] = "Sets the faction discount threshold for this character.",
   },
}

--Set at VARIABLES_LOADED event
local realm = nil
local pName = nil

-- Loading Function --
function FD_AR.OnLoad(self)
   local LOAD_MSG = "Loaded."
   SLASH_FD_AR1="/FDAR"
   SlashCmdList["FD_AR"] = FD_AR.SlashHandler
   FD_AR.SetUpSlashCommands()
   
   FD_AR.HeaderMsg(LOAD_MSG)
   
   self:RegisterEvent("VARIABLES_LOADED")
   
   FD_AR.frame = self
end

-----------------------------------------------------------
-------------------- Slash Handlers =) --------------------
-----------------------------------------------------------
local FD_AR_slash = {}
_G.FD_AR_slash = FD_AR_slash

function FD_AR.SlashHandler(parm)
   if (parm == nil or parm == "") then
      FD_AR.SlashHandler_Default()
   else
      parm = FD_AR.trim(parm)
      local smallParm = strlower(parm)
      
      local slashHandler = FD_AR_slash[smallParm]
      
      if (slashHandler) then
         slashHandler(smallParm)
      else
         FD_AR.HeaderMsg("Unrecognized Slash Command.")
      end
   end
end

function FD_AR.SetUpSlashCommands()
   for key,value in pairs(txt2threshold) do
      FD_AR_slash[key] = FD_AR.SlashHandler_threshold
   end
end

function FD_AR.SlashHandler_Default()
   --list available slash commands
   
   FD_AR.HeaderMsg("Available slash commands:")
   local slash = strlower(SLASH_FD_AR1)
   
   for _,entry in ipairs(slash_subcommands_info) do
      FD_AR.LocalMsg("  ".. slash.." "..entry.name.."   -   "..entry.desc)
   end
end
function FD_AR.SlashHandler_threshold(threshold)
   FD_AR.SetThreshold(txt2threshold[threshold])
   local thresholdText = FD_AR.GetThresholdText()
   FD_AR.HeaderMsg("Threshold now set to |cff00ff00"..thresholdText.."|r.")
end

function FD_AR_slash.enable(parm)
   FD_AR.SetEnabled()
   FD_AR.HeaderMsg("Enabled.")
end
function FD_AR_slash.disable(parm)
   FD_AR.SetDisabled()
   FD_AR.HeaderMsg("Disabled.")
end
function FD_AR_slash.status(parm)
   FD_AR.PrintStatus()
end
-----------------------------------------------------------
-------------------- Event Handlers =) --------------------
-----------------------------------------------------------
local FD_AR_event = {}

function FD_AR_event.VARIABLES_LOADED(...)
   --get saved variable table to local table
   if(_G.FactionDiscount_AutoRepair_saved) then
      FD_AR_saved = _G.FactionDiscount_AutoRepair_saved
   else
      _G.FactionDiscount_AutoRepair_saved = FD_AR_saved
   end
   
   realm = GetRealmName()
   pName = UnitName("player")
   
   FD_AR.ConstructSaveFile()
   
   --this looks weird, but what it's doing is checking if we *should* be enabled, then actually enabling if so
   if( FD_AR.IsEnabled() ) then
      FD_AR.Enable()
   end
end

function FD_AR_event.MERCHANT_SHOW(...)
   -- is merchant a repair merchant of the appropriate rep level?  does the player need a repair?
   if( FD_AR.CheckNPC() and GetRepairAllCost() > 0 ) then
      if ( GetMoney() >= GetRepairAllCost() ) then
         RepairAllItems()
         FD_AR.AnnounceRepaired()
      else
         FD_AR.AnnounceNotEnoughGold()
      end
   end
end
function FD_AR.OnEvent(event, ...)
   if (FD_AR_event[event]) then
      FD_AR_event[event](...)
      return
   end
   
   FD_AR.LocalMsg("Unhandled Event: "..event)
end --FD_AR.OnEvent

-----------------------------------------------------------
------------- Functions for Handling Savefile -------------
-----------------------------------------------------------
function FD_AR.ConstructSaveFile()
   --build data structure
   if not FD_AR_saved then
      FD_AR_saved = {}
   end
   if not FD_AR_saved[realm] then
      FD_AR_saved[realm] = {}
   end
   if not FD_AR_saved[realm][pName] then
      FD_AR_saved[realm][pName] = {}
      FD_AR_saved[realm][pName].enabled = true
      FD_AR_saved[realm][pName].threshold = default_threshold
   end
end
------------------------------------------------------------
------------ Functions for Handling Addon state ------------
------------------------------------------------------------
function FD_AR.IsEnabled()
   return FD_AR_saved[realm][pName].enabled
end

function FD_AR.Enable()
   FD_AR.frame:RegisterEvent("MERCHANT_SHOW")
end
function FD_AR.Disable()
   FD_AR.frame:UnregisterEvent("MERCHANT_SHOW")
end

function FD_AR.SetEnabled()
   FD_AR_saved[realm][pName].enabled = true
   FD_AR.Enable()
end
function FD_AR.SetDisabled()
   FD_AR_saved[realm][pName].enabled = false
   FD_AR.Disable()
end

------------------------------------------------------------
------------- Functions for Handling Threshold -------------
------------------------------------------------------------
function FD_AR.GetThreshold()
   return FD_AR_saved[realm][pName].threshold
end
function FD_AR.SetThreshold(newThreshold)
   FD_AR_saved[realm][pName].threshold = newThreshold
end

function FD_AR.GetThresholdText()
   return threshold2txt[FD_AR.GetThreshold()]
end

-------------------------------------------------------------
------------- Functions for Handling NPC check  -------------
-------------------------------------------------------------
function FD_AR.GetNPCdiscount()
   local discount = UnitReaction("npc", "player")
   return discount
end
function FD_AR.GetNPCdiscountText()
   return discount2txt[FD_AR.GetNPCdiscount()]
end

function FD_AR.CheckNPC()
   --ensure NPC can repair
   if( CanMerchantRepair() ) then
      
      --ensure NPC meets threshold
      local discount = FD_AR.GetNPCdiscount()
      local threshold = FD_AR.GetThreshold()
      
      if ( discount >= threshold ) then return true end
   end
   
   return false
end

------------------------------------------------------------
--- Functions for general handling, not directly related ---
------------------------------------------------------------
function FD_AR.trim(str)
   return (strgsub(str, "^%s*(.-)%s*$", "%1"))
end

-----------------------------------------------------------
------ Functions for showing output on client screen ------
-----------------------------------------------------------
function FD_AR.PrintStatus()
   local enabledText
   if( FD_AR.IsEnabled() ) then
      enabledText = "Enabled"
   else
      enabledText = "Disabled"
   end
   local thresholdText = FD_AR.GetThresholdText()
   
   local announceString = "FDAR is |cff00ff00"..enabledText.."|r and has a faction threshold of: |cff00ff00"..thresholdText.."|r."
   
   FD_AR.HeaderMsg(announceString)
end

function FD_AR.AnnounceRepaired()
   local NPCname = UnitName("npc")
   local discountText = FD_AR.GetNPCdiscountText()
   local cost = GetRepairAllCost() -- in copper
   local costString = ("%dg %ds %dc"):format(cost/10000, cost/100 % 100, cost % 100) -- formatted
   
   local announceString = "Repaired at |cff00ff00"..NPCname.."|r for |cff00ff00"..costString.."|r with a |cff00ff00"..discountText.."|r discount."
   
   FD_AR.HeaderMsg(announceString)
end
function FD_AR.AnnounceNotEnoughGold()
   FD_AR.HeaderMsg("Not enough money to repair.")
end

--the guts
function FD_AR.HeaderMsg(txt)
   FD_AR.LocalMsg("[FactionDiscount |cff00ff00AutoRepair|r] "..txt)
end
function FD_AR.LocalMsg(txt)
   DEFAULT_CHAT_FRAME:AddMessage(txt)
end
